#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasd9_(integer *icompq, integer *ldu, integer *k, real *d__, real *z__, real *vf, real *vl, real *difl, real *difr, real *dsigma, real *work, integer *info);

#ifdef __cplusplus
}
#endif