#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ctrsen_(char *job, char *compq, logical *select, integer *n, complex *t, integer *ldt, complex *q, integer *ldq, complex *w, integer *m, real *s, real *sep, complex *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif