#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ztptrs_(char *uplo, char *trans, char *diag, integer *n, integer *nrhs, doublecomplex *ap, doublecomplex *b, integer *ldb, integer *info);

#ifdef __cplusplus
}
#endif