#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasy2_(logical *ltranl, logical *ltranr, integer *isgn, integer *n1, integer *n2, doublereal *tl, integer *ldtl, doublereal *tr, integer *ldtr, doublereal *b, integer *ldb, doublereal *scale, doublereal *x, integer *ldx, doublereal *xnorm, integer *info);

#ifdef __cplusplus
}
#endif