#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaebz_(integer *ijob, integer *nitmax, integer *n, integer *mmax, integer *minp, integer *nbmin, real *abstol, real *reltol, real *pivmin, real *d__, real *e, real *e2, integer *nval, real *ab, real *c__, integer *mout, integer *nab, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif