#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlanv2_(doublereal *a, doublereal *b, doublereal *c__, doublereal *d__, doublereal *rt1r, doublereal *rt1i, doublereal *rt2r, doublereal *rt2i, doublereal *cs, doublereal *sn);

#ifdef __cplusplus
}
#endif