#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dgbtrf_(integer *m, integer *n, integer *kl, integer *ku, doublereal *ab, integer *ldab, integer *ipiv, integer *info);

#ifdef __cplusplus
}
#endif