#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlarrf_(integer *n, doublereal *d__, doublereal *l, doublereal *ld, doublereal *lld, integer *ifirst, integer *ilast, doublereal *w, doublereal *dplus, doublereal *lplus, doublereal *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif