#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlaein_(logical *rightv, logical *noinit, integer *n, doublecomplex *h__, integer *ldh, doublecomplex *w, doublecomplex *v, doublecomplex *b, integer *ldb, doublereal *rwork, doublereal *eps3, doublereal *smlnum, integer *info);

#ifdef __cplusplus
}
#endif