#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

doublereal zlantr_(char *norm, char *uplo, char *diag, integer *m, integer *n, doublecomplex *a, integer *lda, doublereal *work);

#ifdef __cplusplus
}
#endif