#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasq3_(integer *i0, integer *n0, doublereal *z__, integer *pp, doublereal *dmin__, doublereal *sigma, doublereal *desig, doublereal *qmax, integer *nfail, integer *iter, integer *ndiv, logical *ieee);

#ifdef __cplusplus
}
#endif