#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaein_(logical *rightv, logical *noinit, integer *n, real *h__, integer *ldh, real *wr, real *wi, real *vr, real *vi, real *b, integer *ldb, real *work, real *eps3, real *smlnum, real *bignum, integer *info);

#ifdef __cplusplus
}
#endif