#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasq6_(integer *i0, integer *n0, doublereal *z__, integer *pp, doublereal *dmin__, doublereal *dmin1, doublereal *dmin2, doublereal *dn, doublereal *dnm1, doublereal *dnm2);

#ifdef __cplusplus
}
#endif