#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zhseqr_(char *job, char *compz, integer *n, integer *ilo, integer *ihi, doublecomplex *h__, integer *ldh, doublecomplex *w, doublecomplex *z__, integer *ldz, doublecomplex *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif