#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zpoequ_(integer *n, doublecomplex *a, integer *lda, doublereal *s, doublereal *scond, doublereal *amax, integer *info);

#ifdef __cplusplus
}
#endif