#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dstebz_(char *range, char *order, integer *n, doublereal *vl, doublereal *vu, integer *il, integer *iu, doublereal *abstol, doublereal *d__, doublereal *e, integer *m, integer *nsplit, doublereal *w, integer *iblock, integer *isplit, doublereal *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif