#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int shsein_(char *side, char *eigsrc, char *initv, logical *select, integer *n, real *h__, integer *ldh, real *wr, real *wi, real *vl, integer *ldvl, real *vr, integer *ldvr, integer *mm, integer *m, real *work, integer *ifaill, integer *ifailr, integer *info);

#ifdef __cplusplus
}
#endif