#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int stbcon_(char *norm, char *uplo, char *diag, integer *n, integer *kd, real *ab, integer *ldab, real *rcond, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif