#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasv2_(real *f, real *g, real *h__, real *ssmin, real *ssmax, real *snr, real *csr, real *snl, real *csl);

#ifdef __cplusplus
}
#endif