#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slags2_(logical *upper, real *a1, real *a2, real *a3, real *b1, real *b2, real *b3, real *csu, real *snu, real *csv, real *snv, real *csq, real *snq);

#ifdef __cplusplus
}
#endif