#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlaesy_(doublecomplex *a, doublecomplex *b, doublecomplex *c__, doublecomplex *rt1, doublecomplex *rt2, doublecomplex *evscal, doublecomplex *cs1, doublecomplex *sn1);

#ifdef __cplusplus
}
#endif