#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlarf_(char *side, integer *m, integer *n, doublereal *v, integer *incv, doublereal *tau, doublereal *c__, integer *ldc, doublereal *work);

#ifdef __cplusplus
}
#endif