#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ctgevc_(char *side, char *howmny, logical *select, integer *n, complex *a, integer *lda, complex *b, integer *ldb, complex *vl, integer *ldvl, complex *vr, integer *ldvr, integer *mm, integer *m, complex *work, real *rwork, integer *info);

#ifdef __cplusplus
}
#endif