#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slarrf_(integer *n, real *d__, real *l, real *ld, real *lld, integer *ifirst, integer *ilast, real *w, real *dplus, real *lplus, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif