#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlagtm_(char *trans, integer *n, integer *nrhs, doublereal *alpha, doublereal *dl, doublereal *d__, doublereal *du, doublereal *x, integer *ldx, doublereal *beta, doublereal *b, integer *ldb);

#ifdef __cplusplus
}
#endif