#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlaswp_(integer *n, doublereal *a, integer *lda, integer *k1, integer *k2, integer *ipiv, integer *incx);

#ifdef __cplusplus
}
#endif