#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slarz_(char *side, integer *m, integer *n, integer *l, real *v, integer *incv, real *tau, real *c__, integer *ldc, real *work);

#ifdef __cplusplus
}
#endif