#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slatrd_(char *uplo, integer *n, integer *nb, real *a, integer *lda, real *e, real *tau, real *w, integer *ldw);

#ifdef __cplusplus
}
#endif