#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slagts_(integer *job, integer *n, real *a, real *b, real *c__, real *d__, integer *in, real *y, real *tol, integer *info);

#ifdef __cplusplus
}
#endif