#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zgerqf_(integer *m, integer *n, doublecomplex *a, integer *lda, doublecomplex *tau, doublecomplex *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif