#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlagtf_(integer *n, doublereal *a, doublereal *lambda, doublereal *b, doublereal *c__, doublereal *tol, doublereal *d__, integer *in, integer *info);

#ifdef __cplusplus
}
#endif