#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sbdsdc_(char *uplo, char *compq, integer *n, real *d__, real *e, real *u, integer *ldu, real *vt, integer *ldvt, real *q, integer *iq, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif