#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasr_(char *side, char *pivot, char *direct, integer *m, integer *n, real *c__, real *s, real *a, integer *lda);

#ifdef __cplusplus
}
#endif