#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaeda_(integer *n, integer *tlvls, integer *curlvl, integer *curpbm, integer *prmptr, integer *perm, integer *givptr, integer *givcol, real *givnum, real *q, integer *qptr, real *z__, real *ztemp, integer *info);

#ifdef __cplusplus
}
#endif