#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlasyf_(char *uplo, integer *n, integer *nb, integer *kb, doublecomplex *a, integer *lda, integer *ipiv, doublecomplex *w, integer *ldw, integer *info);

#ifdef __cplusplus
}
#endif