#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sstebz_(char *range, char *order, integer *n, real *vl, real *vu, integer *il, integer *iu, real *abstol, real *d__, real *e, integer *m, integer *nsplit, real *w, integer *iblock, integer *isplit, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif