#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasd9_(integer *icompq, integer *ldu, integer *k, doublereal *d__, doublereal *z__, doublereal *vf, doublereal *vl, doublereal *difl, doublereal *difr, doublereal *dsigma, doublereal *work, integer *info);

#ifdef __cplusplus
}
#endif