#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sopmtr_(char *side, char *uplo, char *trans, integer *m, integer *n, real *ap, real *tau, real *c__, integer *ldc, real *work, integer *info);

#ifdef __cplusplus
}
#endif