#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sgebd2_(integer *m, integer *n, real *a, integer *lda, real *d__, real *e, real *tauq, real *taup, real *work, integer *info);

#ifdef __cplusplus
}
#endif