#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dsbgst_(char *vect, char *uplo, integer *n, integer *ka, integer *kb, doublereal *ab, integer *ldab, doublereal *bb, integer *ldbb, doublereal *x, integer *ldx, doublereal *work, integer *info);

#ifdef __cplusplus
}
#endif