#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlalsd_(char *uplo, integer *smlsiz, integer *n, integer *nrhs, doublereal *d__, doublereal *e, doublereal *b, integer *ldb, doublereal *rcond, integer *rank, doublereal *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif