#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slatzm_(char *side, integer *m, integer *n, real *v, integer *incv, real *tau, real *c1, real *c2, integer *ldc, real *work);

#ifdef __cplusplus
}
#endif