#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlatrd_(char *uplo, integer *n, integer *nb, doublereal *a, integer *lda, doublereal *e, doublereal *tau, doublereal *w, integer *ldw);

#ifdef __cplusplus
}
#endif