#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlarre_(integer *n, doublereal *d__, doublereal *e, doublereal *tol, integer *nsplit, integer *isplit, integer *m, doublereal *w, doublereal *woff, doublereal *gersch, doublereal *work, integer *info);

#ifdef __cplusplus
}
#endif