#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaed9_(integer *k, integer *kstart, integer *kstop, integer *n, real *d__, real *q, integer *ldq, real *rho, real *dlamda, real *w, real *s, integer *lds, integer *info);

#ifdef __cplusplus
}
#endif