#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zgesc2_(integer *n, doublecomplex *a, integer *lda, doublecomplex *rhs, integer *ipiv, integer *jpiv, doublereal *scale);

#ifdef __cplusplus
}
#endif