#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaed0_(integer *icompq, integer *qsiz, integer *n, real *d__, real *e, real *q, integer *ldq, real *qstore, integer *ldqs, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif