#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dgghrd_(char *compq, char *compz, integer *n, integer *ilo, integer *ihi, doublereal *a, integer *lda, doublereal *b, integer *ldb, doublereal *q, integer *ldq, doublereal *z__, integer *ldz, integer *info);

#ifdef __cplusplus
}
#endif