#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasy2_(logical *ltranl, logical *ltranr, integer *isgn, integer *n1, integer *n2, real *tl, integer *ldtl, real *tr, integer *ldtr, real *b, integer *ldb, real *scale, real *x, integer *ldx, real *xnorm, integer *info);

#ifdef __cplusplus
}
#endif