#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasq3_(integer *i0, integer *n0, real *z__, integer *pp, real *dmin__, real *sigma, real *desig, real *qmax, integer *nfail, integer *iter, integer *ndiv, logical *ieee);

#ifdef __cplusplus
}
#endif