#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlargv_(integer *n, doublereal *x, integer *incx, doublereal *y, integer *incy, doublereal *c__, integer *incc);

#ifdef __cplusplus
}
#endif