#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaed6_(integer *kniter, logical *orgati, real *rho, real *d__, real *z__, real *finit, real *tau, integer *info);

#ifdef __cplusplus
}
#endif