#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dbdsdc_(char *uplo, char *compq, integer *n, doublereal *d__, doublereal *e, doublereal *u, integer *ldu, doublereal *vt, integer *ldvt, doublereal *q, integer *iq, doublereal *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif