#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaic1_(integer *job, integer *j, real *x, real *sest, real *w, real *gamma, real *sestpr, real *s, real *c__);

#ifdef __cplusplus
}
#endif