#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int chgeqz_(char *job, char *compq, char *compz, integer *n, integer *ilo, integer *ihi, complex *a, integer *lda, complex *b, integer *ldb, complex *alpha, complex *beta, complex *q, integer *ldq, complex *z__, integer *ldz, complex *work, integer *lwork, real *rwork, integer *info);

#ifdef __cplusplus
}
#endif