#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ztrsyl_(char *trana, char *tranb, integer *isgn, integer *m, integer *n, doublecomplex *a, integer *lda, doublecomplex *b, integer *ldb, doublecomplex *c__, integer *ldc, doublereal *scale, integer *info);

#ifdef __cplusplus
}
#endif