#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlaqtr_(logical *ltran, logical *lreal, integer *n, doublereal *t, integer *ldt, doublereal *b, doublereal *w, doublereal *scale, doublereal *x, doublereal *work, integer *info);

#ifdef __cplusplus
}
#endif