#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ztgex2_(logical *wantq, logical *wantz, integer *n, doublecomplex *a, integer *lda, doublecomplex *b, integer *ldb, doublecomplex *q, integer *ldq, doublecomplex *z__, integer *ldz, integer *j1, integer *info);

#ifdef __cplusplus
}
#endif