#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ztrevc_(char *side, char *howmny, logical *select, integer *n, doublecomplex *t, integer *ldt, doublecomplex *vl, integer *ldvl, doublecomplex *vr, integer *ldvr, integer *mm, integer *m, doublecomplex *work, doublereal *rwork, integer *info);

#ifdef __cplusplus
}
#endif