#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slagv2_(real *a, integer *lda, real *b, integer *ldb, real *alphar, real *alphai, real *beta, real *csl, real *snl, real *csr, real *snr);

#ifdef __cplusplus
}
#endif