#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasv2_(doublereal *f, doublereal *g, doublereal *h__, doublereal *ssmin, doublereal *ssmax, doublereal *snr, doublereal *csr, doublereal *snl, doublereal *csl);

#ifdef __cplusplus
}
#endif