#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dtrsna_(char *job, char *howmny, logical *select, integer *n, doublereal *t, integer *ldt, doublereal *vl, integer *ldvl, doublereal *vr, integer *ldvr, doublereal *s, doublereal *sep, integer *mm, integer *m, doublereal *work, integer *ldwork, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif