#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dspmv_(char *uplo, integer *n, doublereal *alpha, doublereal *ap, doublereal *x, integer *incx, doublereal *beta, doublereal *y, integer *incy);

#ifdef __cplusplus
}
#endif