#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ctgsy2_(char *trans, integer *ijob, integer *m, integer *n, complex *a, integer *lda, complex *b, integer *ldb, complex *c__, integer *ldc, complex *d__, integer *ldd, complex *e, integer *lde, complex *f, integer *ldf, real *scale, real *rdsum, real *rdscal, integer *info);

#ifdef __cplusplus
}
#endif