#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaed1_(integer *n, real *d__, real *q, integer *ldq, integer *indxq, real *rho, integer *cutpnt, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif