#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slahrd_(integer *n, integer *k, integer *nb, real *a, integer *lda, real *tau, real *t, integer *ldt, real *y, integer *ldy);

#ifdef __cplusplus
}
#endif