#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasq4_(integer *i0, integer *n0, doublereal *z__, integer *pp, integer *n0in, doublereal *dmin__, doublereal *dmin1, doublereal *dmin2, doublereal *dn, doublereal *dn1, doublereal *dn2, doublereal *tau, integer *ttype);

#ifdef __cplusplus
}
#endif