#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slanv2_(real *a, real *b, real *c__, real *d__, real *rt1r, real *rt1i, real *rt2r, real *rt2i, real *cs, real *sn);

#ifdef __cplusplus
}
#endif