#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlaqp2_(integer *m, integer *n, integer *offset, doublecomplex *a, integer *lda, integer *jpvt, doublecomplex *tau, doublereal *vn1, doublereal *vn2, doublecomplex *work);

#ifdef __cplusplus
}
#endif