#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zptts2_(integer *iuplo, integer *n, integer *nrhs, doublereal *d__, doublecomplex *e, doublecomplex *b, integer *ldb);

#ifdef __cplusplus
}
#endif