#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sspcon_(char *uplo, integer *n, real *ap, integer *ipiv, real *anorm, real *rcond, real *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif