#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sgtts2_(integer *itrans, integer *n, integer *nrhs, real *dl, real *d__, real *du, real *du2, integer *ipiv, real *b, integer *ldb);

#ifdef __cplusplus
}
#endif