#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlaset_(char *uplo, integer *m, integer *n, doublecomplex *alpha, doublecomplex *beta, doublecomplex *a, integer *lda);

#ifdef __cplusplus
}
#endif