#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasdt_(integer *n, integer *lvl, integer *nd, integer *inode, integer *ndiml, integer *ndimr, integer *msub);

#ifdef __cplusplus
}
#endif