#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int ctpcon_(char *norm, char *uplo, char *diag, integer *n, complex *ap, real *rcond, complex *work, real *rwork, integer *info);

#ifdef __cplusplus
}
#endif