#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasd0_(integer *n, integer *sqre, real *d__, real *e, real *u, integer *ldu, real *vt, integer *ldvt, integer *smlsiz, integer *iwork, real *work, integer *info);

#ifdef __cplusplus
}
#endif