#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlasd1_(integer *nl, integer *nr, integer *sqre, doublereal *d__, doublereal *alpha, doublereal *beta, doublereal *u, integer *ldu, doublereal *vt, integer *ldvt, integer *idxq, integer *iwork, doublereal *work, integer *info);

#ifdef __cplusplus
}
#endif