#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlapll_(integer *n, doublecomplex *x, integer *incx, doublecomplex *y, integer *incy, doublereal *ssmin);

#ifdef __cplusplus
}
#endif