#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dgees_(char *jobvs, char *sort, L_fp select, integer *n, doublereal *a, integer *lda, integer *sdim, doublereal *wr, doublereal *wi, doublereal *vs, integer *ldvs, doublereal *work, integer *lwork, logical *bwork, integer *info);

#ifdef __cplusplus
}
#endif