#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaed2_(integer *k, integer *n, integer *n1, real *d__, real *q, integer *ldq, integer *indxq, real *rho, real *z__, real *dlamda, real *w, real *q2, integer *indx, integer *indxc, integer *indxp, integer *coltyp, integer *info);

#ifdef __cplusplus
}
#endif