#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlatzm_(char *side, integer *m, integer *n, doublecomplex *v, integer *incv, doublecomplex *tau, doublecomplex *c1, doublecomplex *c2, integer *ldc, doublecomplex *work);

#ifdef __cplusplus
}
#endif