#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlaic1_(integer *job, integer *j, doublecomplex *x, doublereal *sest, doublecomplex *w, doublecomplex *gamma, doublereal *sestpr, doublecomplex *s, doublecomplex *c__);

#ifdef __cplusplus
}
#endif