#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlaqsp_(char *uplo, integer *n, doublereal *ap, doublereal *s, doublereal *scond, doublereal *amax, char *equed);

#ifdef __cplusplus
}
#endif