#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int claqge_(integer *m, integer *n, complex *a, integer *lda, real *r__, real *c__, real *rowcnd, real *colcnd, real *amax, char *equed);

#ifdef __cplusplus
}
#endif