#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sgebal_(char *job, integer *n, real *a, integer *lda, integer *ilo, integer *ihi, real *scale, integer *info);

#ifdef __cplusplus
}
#endif