#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zggglm_(integer *n, integer *m, integer *p, doublecomplex *a, integer *lda, doublecomplex *b, integer *ldb, doublecomplex *d__, doublecomplex *x, doublecomplex *y, doublecomplex *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif