#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sgeev_(char *jobvl, char *jobvr, integer *n, real *a, integer *lda, real *wr, real *wi, real *vl, integer *ldvl, real *vr, integer *ldvr, real *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif