#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slaexc_(logical *wantq, integer *n, real *t, integer *ldt, real *q, integer *ldq, integer *j1, integer *n1, integer *n2, real *work, integer *info);

#ifdef __cplusplus
}
#endif