#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int chetd2_(char *uplo, integer *n, complex *a, integer *lda, real *d__, real *e, complex *tau, integer *info);

#ifdef __cplusplus
}
#endif