#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int sptrfs_(integer *n, integer *nrhs, real *d__, real *e, real *df, real *ef, real *b, integer *ldb, real *x, integer *ldx, real *ferr, real *berr, real *work, integer *info);

#ifdef __cplusplus
}
#endif