#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlarrb_(integer *n, doublereal *d__, doublereal *l, doublereal *ld, doublereal *lld, integer *ifirst, integer *ilast, doublereal *sigma, doublereal *reltol, doublereal *w, doublereal *wgap, doublereal *werr, doublereal *work, integer *iwork, integer *info);

#ifdef __cplusplus
}
#endif