#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int zlacrt_(integer *n, doublecomplex *cx, integer *incx, doublecomplex *cy, integer *incy, doublecomplex *c__, doublecomplex *s);

#ifdef __cplusplus
}
#endif