#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int slasd4_(integer *n, integer *i__, real *d__, real *z__, real *delta, real *rho, real *sigma, real *work, integer *info);

#ifdef __cplusplus
}
#endif