#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int shseqr_(char *job, char *compz, integer *n, integer *ilo, integer *ihi, real *h__, integer *ldh, real *wr, real *wi, real *z__, integer *ldz, real *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif