#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlaic1_(integer *job, integer *j, doublereal *x, doublereal *sest, doublereal *w, doublereal *gamma, doublereal *sestpr, doublereal *s, doublereal *c__);

#ifdef __cplusplus
}
#endif