#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dtgex2_(logical *wantq, logical *wantz, integer *n, doublereal *a, integer *lda, doublereal *b, integer *ldb, doublereal *q, integer *ldq, doublereal *z__, integer *ldz, integer *j1, integer *n1, integer *n2, doublereal *work, integer *lwork, integer *info);

#ifdef __cplusplus
}
#endif